/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

"use strict";

var Cc = Components.classes;
var Ci = Components.interfaces;
var Cu = Components.utils;

Cu.import("resource://gre/modules/Services.jsm");

var m_isInstall = false;

// install & uninstall are called even for disabled extensions
function install(data, reason) {
  m_isInstall = true;
}


function uninstall(data, reason) {
  registerResourceProtocol(data.resourceURI);

  // uninstall: remove storage, prefs
  if (reason === ADDON_UNINSTALL) { // updating=ADDON_UPGRADE
    Bootstrap.extensionUninstall();
  }

  // update/uninstall: unload all modules
  registerResourceProtocol(null);
}


function startup(data, reason) {
  // resource protocol needed because when mm.broadcastAsyncMessage("bbb@hultmann:disable")
  // listener executes chrome protocol doesn't exist anymore
  registerResourceProtocol(data.resourceURI);
  Bootstrap.extensionStartup(m_isInstall);
}


function shutdown(data, reason) {
  if (reason !== APP_SHUTDOWN) {
    Bootstrap.extensionShutdown();
  }
}


// chrome.manifest line:
// resource ext-modules modules/
function registerResourceProtocol(uri) { // null to unregister
  var module = uri ? Services.io.newURI(uri.resolve("content/"), null, null) : null;
  Services.io.getProtocolHandler("resource").
                QueryInterface(Ci.nsIResProtocolHandler).
                  setSubstitution("big-block-brasil", module);
}


var Bootstrap = {
  extensionStartup: function(isInstall) {
    if (isInstall) {
      console.log("bbb isInstall setting prefs");
      this._setDefaultPrefs();
    }
    Services.mm.loadFrameScript("resource://big-block-brasil/content.js?v=1.0.1", true);
  },


  _setDefaultPrefs: function() {
    var p = Services.prefs;
    function setBool(name, val) {
      if (p.getPrefType(name) !== p.PREF_BOOL) {
        p.setBoolPref(name, val);
      }
    }

    setBool("services.sync.prefs.sync.extensions.bbb@hultmann.blockmode", true);
    setBool("services.sync.prefs.sync.extensions.bbb@hultmann.social", true);
    setBool("services.sync.prefs.sync.extensions.bbb@hultmann.news", true);
    setBool("services.sync.prefs.sync.extensions.bbb@hultmann.allsites", true);

    if (p.getPrefType("extensions.bbb@hultmann.blockmode") !== p.PREF_STRING) {
      p.setCharPref("extensions.bbb@hultmann.blockmode", "hide");
    }

    setBool("extensions.bbb@hultmann.social", true);
    setBool("extensions.bbb@hultmann.news", true);
    setBool("extensions.bbb@hultmann.allsites", true);
  },


  extensionShutdown: function() {
    Services.mm.addMessageListener("bbb@hultmann:content-disabled", function unload(message) {
      Services.mm.removeMessageListener("bbb@hultmann:content-disabled", unload);
      registerResourceProtocol(null);
      console.log("BBB chrome disable done!");
    });
    mm.removeDelayedFrameScript("resource://big-block-brasil/content.js?v=1.0.1");
    mm.broadcastAsyncMessage("bbb@hultmann:disable");
  },


  extensionUninstall: function() {
    var branch = "extensions.bbb@hultmann.";
    Services.prefs.getBranch(branch).deleteBranch("");
    Services.prefs.getBranch("services.sync.prefs.sync." + branch).deleteBranch("");
  }
};
